@@ -1,4 +1,5 @@
 //! A node which represents a subtree of a patricia tree.
+use crate::BorrowedBytes;
 use std::alloc::{alloc, dealloc, handle_alloc_error, Layout};
 use std::marker::PhantomData;
 use std::mem;
@@ -16,6 +17,7 @@ macro_rules! assert_some {
 }
 
 bitflags! {
+    #[derive(Clone, Copy)]
     pub (crate) struct Flags: u8 {
         const VALUE_ALLOCATED = 0b0000_0001;
         const VALUE_INITIALIZED = 0b0000_0010;
@@ -28,9 +30,6 @@ bitflags! {
     }
 }
 
-#[cfg(feature = "binary-format")]
-pub use crate::codec::{NodeDecoder, NodeEncoder};
-
 const FLAGS_OFFSET: isize = 0;
 const LABEL_LEN_OFFSET: isize = 1;
 const LABEL_OFFSET: isize = 2;
@@ -58,51 +57,11 @@ unsafe impl<V: Send> Send for Node<V> {}
 unsafe impl<V: Sync> Sync for Node<V> {}
 impl<V> Node<V> {
     /// Makes a new node which represents an empty tree.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use patricia_tree::node::Node;
-    ///
-    /// let node = Node::<()>::root();
-    /// assert!(node.label().is_empty());
-    /// assert!(node.value().is_none());
-    /// assert!(node.child().is_none());
-    /// assert!(node.sibling().is_none());
-    /// ```
     pub fn root() -> Self {
         Node::new(b"", None, None, None)
     }
 
     /// Makes a new node.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use patricia_tree::node::Node;
-    ///
-    /// let node0 = Node::new("foo".as_ref(), Some(3), None, None);
-    /// assert_eq!(node0.label(), b"foo");
-    /// assert_eq!(node0.value(), Some(&3));
-    /// assert_eq!(node0.child().map(|n| n.label()), None);
-    /// assert_eq!(node0.sibling().map(|n| n.label()), None);
-    ///
-    /// let node1 = Node::new("bar".as_ref(), None, None, Some(node0));
-    /// assert_eq!(node1.label(), b"bar");
-    /// assert_eq!(node1.value(), None);
-    /// assert_eq!(node1.child().map(|n| n.label()), None);
-    /// assert_eq!(node1.sibling().map(|n| n.label()), Some(&b"foo"[..]));
-    ///
-    /// // If the length of a label name is longer than 255, it will be splitted to two nodes.
-    /// let node2 = Node::new([b'a'; 256].as_ref(), Some(4), Some(node1), None);
-    /// assert_eq!(node2.label(), [b'a'; 255].as_ref());
-    /// assert_eq!(node2.value(), None);
-    /// assert_eq!(node2.child().map(|n| n.label()), Some(&b"a"[..]));
-    /// assert_eq!(node2.sibling().map(|n| n.label()), None);
-    ///
-    /// assert_eq!(node2.child().unwrap().value(), Some(&4));
-    /// assert_eq!(node2.child().unwrap().child().unwrap().label(), b"bar");
-    /// ```
     pub fn new(
         mut label: &[u8],
         mut value: Option<V>,
@@ -142,7 +101,7 @@ impl<V> Node<V> {
                 handle_alloc_error(layout)
             }
 
-            ptr::write(ptr.offset(FLAGS_OFFSET), flags.bits() as u8);
+            ptr::write(ptr.offset(FLAGS_OFFSET), flags.bits());
             ptr::write(ptr.offset(LABEL_LEN_OFFSET), label.len() as u8);
             ptr::copy_nonoverlapping(label.as_ptr(), ptr.offset(LABEL_OFFSET), label.len());
 
@@ -162,7 +121,7 @@ impl<V> Node<V> {
         }
     }
 
-    #[cfg(feature = "binary-format")]
+    #[cfg(feature = "serde")]
     pub(crate) fn new_for_decoding(flags: Flags, label_len: u8) -> Self {
         let mut init_flags = Flags::empty();
         let mut layout = Self::initial_layout(label_len as usize);
@@ -200,7 +159,7 @@ impl<V> Node<V> {
         }
     }
 
-    #[cfg(feature = "binary-format")]
+    #[cfg(feature = "serde")]
     pub(crate) fn label_mut(&mut self) -> &mut [u8] {
         unsafe {
             let label_len = *self.ptr.offset(LABEL_LEN_OFFSET) as usize;
@@ -412,29 +371,6 @@ impl<V> Node<V> {
     }
 
     /// Gets an iterator which traverses the nodes in this tree, in depth first order.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use patricia_tree::PatriciaSet;
-    /// use patricia_tree::node::Node;
-    ///
-    /// let mut set = PatriciaSet::new();
-    /// set.insert("foo");
-    /// set.insert("bar");
-    /// set.insert("baz");
-    ///
-    /// let node = Node::from(set);
-    /// let nodes = node.iter().map(|(level, node)| (level, node.label())).collect::<Vec<_>>();
-    /// assert_eq!(nodes,
-    ///            [
-    ///                (0, "".as_ref()),
-    ///                (1, "ba".as_ref()),
-    ///                (2, "r".as_ref()),
-    ///                (2, "z".as_ref()),
-    ///                (1, "foo".as_ref())
-    ///            ]);
-    /// ```
     pub fn iter(&self) -> Iter<V> {
         Iter {
             stack: vec![(0, self)],
@@ -442,29 +378,6 @@ impl<V> Node<V> {
     }
 
     /// Gets a mutable iterator which traverses the nodes in this tree, in depth first order.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use patricia_tree::PatriciaSet;
-    /// use patricia_tree::node::Node;
-    ///
-    /// let mut set = PatriciaSet::new();
-    /// set.insert("foo");
-    /// set.insert("bar");
-    /// set.insert("baz");
-    ///
-    /// let mut node = Node::from(set);
-    /// let nodes = node.iter_mut().map(|(level, node)| (level, node.label())).collect::<Vec<_>>();
-    /// assert_eq!(nodes,
-    ///            [
-    ///                (0, "".as_ref()),
-    ///                (1, "ba".as_ref()),
-    ///                (2, "r".as_ref()),
-    ///                (2, "z".as_ref()),
-    ///                (1, "foo".as_ref())
-    ///            ]);
-    /// ```
     pub fn iter_mut(&mut self) -> IterMut<V> {
         IterMut {
             stack: vec![(0, self)],
@@ -483,9 +396,12 @@ impl<V> Node<V> {
         }
     }
 
-    pub(crate) fn common_prefixes<K>(&self, key: K) -> CommonPrefixesIter<K, V>
+    pub(crate) fn common_prefixes<'a, 'b, K>(
+        &'a self,
+        key: &'b K,
+    ) -> CommonPrefixesIter<'a, 'b, K, V>
     where
-        K: AsRef<[u8]>,
+        K: ?Sized + BorrowedBytes,
     {
         CommonPrefixesIter {
             key,
@@ -493,43 +409,64 @@ impl<V> Node<V> {
         }
     }
 
-    pub(crate) fn get(&self, key: &[u8]) -> Option<&V> {
-        let common_prefix_len = self.skip_common_prefix(key);
-        let next = &key[common_prefix_len..];
+    pub(crate) fn get<K: ?Sized + BorrowedBytes>(&self, key: &K) -> Option<&V> {
+        let (next, common_prefix_len) = key.strip_common_prefix_and_len(self.label());
         if common_prefix_len == self.label().len() {
             if next.is_empty() {
                 self.value()
             } else {
                 self.child().and_then(|child| child.get(next))
             }
-        } else if common_prefix_len == 0 && self.label().get(0) <= key.get(0) {
+        } else if common_prefix_len == 0 && key.cmp_first_item(self.label()).is_ge() {
             self.sibling().and_then(|sibling| sibling.get(next))
         } else {
             None
         }
     }
-    pub(crate) fn get_mut(&mut self, key: &[u8]) -> Option<&mut V> {
-        let common_prefix_len = self.skip_common_prefix(key);
-        let next = &key[common_prefix_len..];
+
+    pub(crate) fn get_mut<K: ?Sized + BorrowedBytes>(&mut self, key: &K) -> Option<&mut V> {
+        let (next, common_prefix_len) = key.strip_common_prefix_and_len(self.label());
         if common_prefix_len == self.label().len() {
             if next.is_empty() {
                 self.value_mut()
             } else {
                 self.child_mut().and_then(|child| child.get_mut(next))
             }
-        } else if common_prefix_len == 0 && self.label().get(0) <= key.get(0) {
+        } else if common_prefix_len == 0 && key.cmp_first_item(self.label()).is_ge() {
             self.sibling_mut().and_then(|sibling| sibling.get_mut(next))
         } else {
             None
         }
     }
-    pub(crate) fn get_longest_common_prefix(
+    pub(crate) fn longest_common_prefix_len<K: ?Sized + BorrowedBytes>(
         &self,
-        key: &[u8],
+        key: &K,
+        offset: usize,
+    ) -> usize {
+        let (next, common_prefix_len) = key.strip_common_prefix_and_len(self.label());
+        let next_offset = offset + common_prefix_len;
+        if common_prefix_len == self.label().len() {
+            if next.is_empty() {
+                next_offset
+            } else {
+                self.child()
+                    .map(|child| child.longest_common_prefix_len(next, next_offset))
+                    .unwrap_or(next_offset)
+            }
+        } else if common_prefix_len == 0 && key.cmp_first_item(self.label()).is_ge() {
+            self.sibling()
+                .map(|sibling| sibling.longest_common_prefix_len(next, offset))
+                .unwrap_or(next_offset)
+        } else {
+            next_offset
+        }
+    }
+    pub(crate) fn get_longest_common_prefix<K: ?Sized + BorrowedBytes>(
+        &self,
+        key: &K,
         offset: usize,
     ) -> Option<(usize, &V)> {
-        let common_prefix_len = self.skip_common_prefix(key);
-        let next = &key[common_prefix_len..];
+        let (next, common_prefix_len) = key.strip_common_prefix_and_len(self.label());
         if common_prefix_len == self.label().len() {
             let offset = offset + common_prefix_len;
             if next.is_empty() {
@@ -539,55 +476,79 @@ impl<V> Node<V> {
                     .and_then(|child| child.get_longest_common_prefix(next, offset))
                     .or_else(|| self.value().map(|v| (offset, v)))
             }
-        } else if common_prefix_len == 0 && self.label().get(0) <= key.get(0) {
+        } else if common_prefix_len == 0 && key.cmp_first_item(self.label()).is_ge() {
             self.sibling()
                 .and_then(|sibling| sibling.get_longest_common_prefix(next, offset))
         } else {
             None
         }
     }
-
-    pub(crate) fn get_prefix_node(&self, key: &[u8], offset: usize) -> Option<(usize, &Self)> {
-        let common_prefix_len = self.skip_common_prefix(key);
-        let next = &key[common_prefix_len..];
-        if next.is_empty() {
-            Some((common_prefix_len, self))
-        } else if common_prefix_len == self.label().len() {
-            let offset = offset + common_prefix_len;
-            self.child()
-                .and_then(|child| child.get_prefix_node(next, offset))
-        } else if common_prefix_len == 0 && self.label().get(0) <= key.get(0) {
-            self.sibling()
-                .and_then(|sibling| sibling.get_prefix_node(next, offset))
-        } else {
-            None
-        }
-    }
-
-    pub(crate) fn get_prefix_node_mut(
+    pub(crate) fn get_longest_common_prefix_mut<K: ?Sized + BorrowedBytes>(
         &mut self,
-        key: &[u8],
+        key: &K,
         offset: usize,
-    ) -> Option<(usize, &mut Self)> {
-        let common_prefix_len = self.skip_common_prefix(key);
-        let next = &key[common_prefix_len..];
-        if next.is_empty() {
-            Some((common_prefix_len, self))
-        } else if common_prefix_len == self.label().len() {
+    ) -> Option<(usize, &mut V)> {
+        let (next, common_prefix_len) = key.strip_common_prefix_and_len(self.label());
+        if common_prefix_len == self.label().len() {
             let offset = offset + common_prefix_len;
-            self.child_mut()
-                .and_then(|child| child.get_prefix_node_mut(next, offset))
-        } else if common_prefix_len == 0 && self.label().get(0) <= key.get(0) {
+            if next.is_empty() {
+                self.value_mut().map(|v| (offset, v))
+            } else {
+                let this = self.as_mut();
+                this.child
+                    .and_then(|child| child.get_longest_common_prefix_mut(next, offset))
+                    .or_else(|| this.value.map(|v| (offset, v)))
+            }
+        } else if common_prefix_len == 0 && key.cmp_first_item(self.label()).is_ge() {
             self.sibling_mut()
-                .and_then(|sibling| sibling.get_prefix_node_mut(next, offset))
+                .and_then(|sibling| sibling.get_longest_common_prefix_mut(next, offset))
         } else {
             None
         }
     }
 
-    pub(crate) fn split_by_prefix(&mut self, prefix: &[u8]) -> Option<Self> {
-        let common_prefix_len = self.skip_common_prefix(prefix);
-        if common_prefix_len == prefix.len() {
+    pub(crate) fn get_prefix_node<K: ?Sized + BorrowedBytes>(
+        &self,
+        key: &K,
+    ) -> Option<(usize, &Self)> {
+        let (next, common_prefix_len) = key.strip_common_prefix_and_len(self.label());
+        if next.is_empty() {
+            Some((common_prefix_len, self))
+        } else if common_prefix_len == self.label().len() {
+            self.child().and_then(|child| child.get_prefix_node(next))
+        } else if common_prefix_len == 0 && key.cmp_first_item(self.label()).is_ge() {
+            self.sibling()
+                .and_then(|sibling| sibling.get_prefix_node(next))
+        } else {
+            None
+        }
+    }
+
+    pub(crate) fn get_prefix_node_mut<K: ?Sized + BorrowedBytes>(
+        &mut self,
+        key: &K,
+    ) -> Option<(usize, &mut Self)> {
+        let (next, common_prefix_len) = key.strip_common_prefix_and_len(self.label());
+        if next.is_empty() {
+            Some((common_prefix_len, self))
+        } else if common_prefix_len == self.label().len() {
+            self.child_mut()
+                .and_then(|child| child.get_prefix_node_mut(next))
+        } else if common_prefix_len == 0 && key.cmp_first_item(self.label()).is_ge() {
+            self.sibling_mut()
+                .and_then(|sibling| sibling.get_prefix_node_mut(next))
+        } else {
+            None
+        }
+    }
+
+    pub(crate) fn split_by_prefix<K: ?Sized + BorrowedBytes>(
+        &mut self,
+        prefix: &K,
+        level: usize,
+    ) -> Option<Self> {
+        let (next, common_prefix_len) = prefix.strip_common_prefix_and_len(self.label());
+        if common_prefix_len == prefix.as_bytes().len() {
             let value = self.take_value();
             let child = self.take_child();
             let node = Node::new(&self.label()[common_prefix_len..], value, child, None);
@@ -596,18 +557,16 @@ impl<V> Node<V> {
             }
             Some(node)
         } else if common_prefix_len == self.label().len() {
-            let next = &prefix[common_prefix_len..];
             self.child_mut()
-                .and_then(|child| child.split_by_prefix(next))
+                .and_then(|child| child.split_by_prefix(next, level + 1))
                 .map(|old| {
                     self.try_reclaim_child();
-                    self.try_merge_with_child();
+                    self.try_merge_with_child(level);
                     old
                 })
-        } else if common_prefix_len == 0 && self.label().get(0) <= prefix.get(0) {
-            let next = &prefix[common_prefix_len..];
+        } else if common_prefix_len == 0 && prefix.cmp_first_item(self.label()).is_ge() {
             self.sibling_mut()
-                .and_then(|sibling| sibling.split_by_prefix(next))
+                .and_then(|sibling| sibling.split_by_prefix(next, level))
                 .map(|old| {
                     self.try_reclaim_sibling();
                     old
@@ -616,27 +575,26 @@ impl<V> Node<V> {
             None
         }
     }
-    pub(crate) fn remove(&mut self, key: &[u8]) -> Option<V> {
-        let common_prefix_len = self.skip_common_prefix(key);
-        let next = &key[common_prefix_len..];
+    pub(crate) fn remove<K: ?Sized + BorrowedBytes>(&mut self, key: &K, level: usize) -> Option<V> {
+        let (next, common_prefix_len) = key.strip_common_prefix_and_len(self.label());
         if common_prefix_len == self.label().len() {
             if next.is_empty() {
                 self.take_value().map(|old| {
-                    self.try_merge_with_child();
+                    self.try_merge_with_child(level);
                     old
                 })
             } else {
                 self.child_mut()
-                    .and_then(|child| child.remove(next))
+                    .and_then(|child| child.remove(next, level + 1))
                     .map(|old| {
                         self.try_reclaim_child();
-                        self.try_merge_with_child();
+                        self.try_merge_with_child(level);
                         old
                     })
             }
-        } else if common_prefix_len == 0 && self.label().get(0) <= key.get(0) {
+        } else if common_prefix_len == 0 && key.cmp_first_item(self.label()).is_ge() {
             self.sibling_mut()
-                .and_then(|sibling| sibling.remove(next))
+                .and_then(|sibling| sibling.remove(next, level))
                 .map(|old| {
                     self.try_reclaim_sibling();
                     old
@@ -645,22 +603,21 @@ impl<V> Node<V> {
             None
         }
     }
-    pub(crate) fn insert(&mut self, key: &[u8], value: V) -> Option<V> {
-        if self.label().get(0) > key.get(0) {
+    pub(crate) fn insert<K: ?Sized + BorrowedBytes>(&mut self, key: &K, value: V) -> Option<V> {
+        if key.cmp_first_item(self.label()).is_lt() {
             let this = Node {
                 ptr: self.ptr,
                 _value: PhantomData,
             };
-            let node = Node::new(key, Some(value), None, Some(this));
+            let node = Node::new(key.as_bytes(), Some(value), None, Some(this));
             self.ptr = node.ptr;
             mem::forget(node);
             return None;
         }
 
-        let common_prefix_len = self.skip_common_prefix(key);
-        let next = &key[common_prefix_len..];
+        let (next, common_prefix_len) = key.strip_common_prefix_and_len(self.label());
         let is_label_matched = common_prefix_len == self.label().len();
-        if next.is_empty() {
+        if next.as_bytes().is_empty() {
             if is_label_matched {
                 let old = self.take_value();
                 self.set_value(value);
@@ -674,14 +631,14 @@ impl<V> Node<V> {
             if let Some(child) = self.child_mut() {
                 return child.insert(next, value);
             }
-            let child = Node::new(next, Some(value), None, None);
+            let child = Node::new(next.as_bytes(), Some(value), None, None);
             self.set_child(child);
             None
         } else if common_prefix_len == 0 {
             if let Some(sibling) = self.sibling_mut() {
                 return sibling.insert(next, value);
             }
-            let sibling = Node::new(next, Some(value), None, None);
+            let sibling = Node::new(next.as_bytes(), Some(value), None, None);
             self.set_sibling(sibling);
             None
         } else {
@@ -690,21 +647,13 @@ impl<V> Node<V> {
             None
         }
     }
-
-    fn skip_common_prefix(&self, key: &[u8]) -> usize {
-        self.label()
-            .iter()
-            .zip(key.iter())
-            .take_while(|x| x.0 == x.1)
-            .count()
-    }
     pub(crate) fn flags(&self) -> Flags {
         Flags::from_bits_truncate(unsafe { *self.ptr })
     }
     fn set_flags(&mut self, other: Flags, value: bool) {
         let mut flags = self.flags();
         flags.set(other, value);
-        unsafe { ptr::write(self.ptr, flags.bits() as u8) };
+        unsafe { ptr::write(self.ptr, flags.bits()) };
     }
     fn label_len(&self) -> usize {
         unsafe { *self.ptr.offset(LABEL_LEN_OFFSET) as usize }
@@ -776,7 +725,11 @@ impl<V> Node<V> {
             self.set_child(child);
         }
     }
-    pub(crate) fn try_merge_with_child(&mut self) {
+    pub(crate) fn try_merge_with_child(&mut self, level: usize) {
+        if level == 0 {
+            return;
+        }
+
         if self.flags().contains(Flags::VALUE_INITIALIZED)
             || !self.flags().contains(Flags::CHILD_INITIALIZED)
         {
@@ -888,41 +841,11 @@ pub struct NodeMut<'a, V: 'a> {
     child: Option<&'a mut Node<V>>,
 }
 impl<'a, V: 'a> NodeMut<'a, V> {
-    /// Makes a new reference to a node's label and mutable value.
-    pub fn new(
-        label: &'a [u8],
-        value: Option<&'a mut V>,
-        sibling: Option<&'a mut Node<V>>,
-        child: Option<&'a mut Node<V>>,
-    ) -> Self {
-        NodeMut {
-            label,
-            value,
-            sibling,
-            child,
-        }
-    }
-
     /// Returns the label of the node.
     pub fn label(&self) -> &'a [u8] {
         self.label
     }
 
-    /// Returns the value of the node, if present.
-    pub fn value(&self) -> &Option<&'a mut V> {
-        &self.value
-    }
-
-    /// Returns the sibling of the node, if present.
-    pub fn sibling(&self) -> &Option<&'a mut Node<V>> {
-        &self.sibling
-    }
-
-    /// Returns the child of the node, if present.
-    pub fn child(&self) -> &Option<&'a mut Node<V>> {
-        &self.child
-    }
-
     /// Converts into a mutable reference to the value.
     pub fn into_value_mut(self) -> Option<&'a mut V> {
         self.value
@@ -952,20 +875,21 @@ impl<'a, V: 'a> Iterator for IterMut<'a, V> {
 /// An iterator over entries in that collects all values up to
 /// until the key stops matching.
 #[derive(Debug)]
-pub(crate) struct CommonPrefixesIter<'a, K, V> {
-    key: K,
+pub(crate) struct CommonPrefixesIter<'a, 'b, K: ?Sized, V> {
+    key: &'b K,
     stack: Vec<(usize, &'a Node<V>)>,
 }
 
-impl<'a, K, V> Iterator for CommonPrefixesIter<'a, K, V>
+impl<'a, 'b, K, V> Iterator for CommonPrefixesIter<'a, 'b, K, V>
 where
-    K: AsRef<[u8]>,
+    K: ?Sized + BorrowedBytes,
 {
     type Item = (usize, &'a Node<V>);
     fn next(&mut self) -> Option<Self::Item> {
         while let Some((offset, node)) = self.stack.pop() {
-            let common_prefix_len = node.skip_common_prefix(&self.key.as_ref()[offset..]);
-            if common_prefix_len == 0 && node.label().get(0) <= self.key.as_ref().get(offset) {
+            let key = self.key.strip_n_prefix(offset);
+            let (_next, common_prefix_len) = key.strip_common_prefix_and_len(node.label());
+            if common_prefix_len == 0 && key.cmp_first_item(node.label()).is_ge() {
                 if let Some(sibling) = node.sibling() {
                     self.stack.push((offset, sibling));
                 }
@@ -1010,9 +934,91 @@ impl<V> Iterator for IntoIter<V> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::PatriciaSet;
+    use crate::{PatriciaSet, StringPatriciaMap};
     use std::str;
 
+    #[test]
+    fn root_works() {
+        let node = Node::<()>::root();
+        assert!(node.label().is_empty());
+        assert!(node.value().is_none());
+        assert!(node.child().is_none());
+        assert!(node.sibling().is_none());
+    }
+
+    #[test]
+    fn new_works() {
+        let node0 = Node::new("foo".as_ref(), Some(3), None, None);
+        assert_eq!(node0.label(), b"foo");
+        assert_eq!(node0.value(), Some(&3));
+        assert_eq!(node0.child().map(|n| n.label()), None);
+        assert_eq!(node0.sibling().map(|n| n.label()), None);
+
+        let node1 = Node::new("bar".as_ref(), None, None, Some(node0));
+        assert_eq!(node1.label(), b"bar");
+        assert_eq!(node1.value(), None);
+        assert_eq!(node1.child().map(|n| n.label()), None);
+        assert_eq!(node1.sibling().map(|n| n.label()), Some(&b"foo"[..]));
+
+        // If the length of a label name is longer than 255, it will be splitted to two nodes.
+        let node2 = Node::new([b'a'; 256].as_ref(), Some(4), Some(node1), None);
+        assert_eq!(node2.label(), [b'a'; 255].as_ref());
+        assert_eq!(node2.value(), None);
+        assert_eq!(node2.child().map(|n| n.label()), Some(&b"a"[..]));
+        assert_eq!(node2.sibling().map(|n| n.label()), None);
+
+        assert_eq!(node2.child().unwrap().value(), Some(&4));
+        assert_eq!(node2.child().unwrap().child().unwrap().label(), b"bar");
+    }
+
+    #[test]
+    fn ietr_works() {
+        let mut set = PatriciaSet::new();
+        set.insert("foo");
+        set.insert("bar");
+        set.insert("baz");
+
+        let root = set.into_node();
+        let nodes = root
+            .iter()
+            .map(|(level, node)| (level, node.label()))
+            .collect::<Vec<_>>();
+        assert_eq!(
+            nodes,
+            [
+                (0, "".as_ref()),
+                (1, "ba".as_ref()),
+                (2, "r".as_ref()),
+                (2, "z".as_ref()),
+                (1, "foo".as_ref())
+            ]
+        );
+    }
+
+    #[test]
+    fn iter_mut_works() {
+        let mut set = PatriciaSet::new();
+        set.insert("foo");
+        set.insert("bar");
+        set.insert("baz");
+
+        let mut root = set.into_node();
+        let nodes = root
+            .iter_mut()
+            .map(|(level, node)| (level, node.label()))
+            .collect::<Vec<_>>();
+        assert_eq!(
+            nodes,
+            [
+                (0, "".as_ref()),
+                (1, "ba".as_ref()),
+                (2, "r".as_ref()),
+                (2, "z".as_ref()),
+                (1, "foo".as_ref())
+            ]
+        );
+    }
+
     #[test]
     fn long_label_works() {
         let node = Node::new(&[b'a'; 256][..], Some(10), None, None);
@@ -1036,13 +1042,16 @@ mod tests {
         );
 
         set.remove("123def");
-        assert_eq!(set_to_labels(&set), [(0, "123"), (1, "456"), (1, "abc")]);
+        assert_eq!(
+            set_to_labels(&set),
+            [(0, ""), (1, "123"), (2, "456"), (2, "abc")]
+        );
 
         set.remove("123456");
-        assert_eq!(set_to_labels(&set), [(0, "123"), (1, "abc")]);
+        assert_eq!(set_to_labels(&set), [(0, ""), (1, "123"), (2, "abc")]);
 
         set.remove("123");
-        assert_eq!(set_to_labels(&set), [(0, "123abc")]);
+        assert_eq!(set_to_labels(&set), [(0, ""), (1, "123abc")]);
     }
 
     #[test]
@@ -1061,8 +1070,48 @@ mod tests {
         assert_eq!(lcp("123456789"), Some("123456".as_bytes()));
     }
 
+    #[test]
+    fn get_longest_common_prefix_mut_works() {
+        let mut map = [
+            ("123", 1),
+            ("123456", 2),
+            ("1234_67", 3),
+            ("123abc", 4),
+            ("123def", 5),
+        ]
+        .iter()
+        .cloned()
+        .map(|(k, v)| (String::from(k), v))
+        .collect::<StringPatriciaMap<usize>>();
+
+        assert_eq!(map.get_longest_common_prefix_mut(""), None);
+        assert_eq!(map.get_longest_common_prefix_mut("12"), None);
+        assert_eq!(
+            map.get_longest_common_prefix_mut("123"),
+            Some(("123", &mut 1))
+        );
+        *map.get_longest_common_prefix_mut("123").unwrap().1 = 10;
+        assert_eq!(
+            map.get_longest_common_prefix_mut("1234"),
+            Some(("123", &mut 10))
+        );
+        assert_eq!(
+            map.get_longest_common_prefix_mut("123456"),
+            Some(("123456", &mut 2))
+        );
+        *map.get_longest_common_prefix_mut("1234567").unwrap().1 = 20;
+        assert_eq!(
+            map.get_longest_common_prefix_mut("1234_6"),
+            Some(("123", &mut 10))
+        );
+        assert_eq!(
+            map.get_longest_common_prefix_mut("123456789"),
+            Some(("123456", &mut 20))
+        );
+    }
+
     fn set_to_labels(set: &PatriciaSet) -> Vec<(usize, &str)> {
-        set.as_ref()
+        set.as_node()
             .iter()
             .map(|(level, n)| (level, str::from_utf8(n.label()).unwrap()))
             .collect()
